( cd usr/lib ; rm -rf librom1394.so.0 )
( cd usr/lib ; ln -sf librom1394.so.0.3.0 librom1394.so.0 )
( cd usr/lib ; rm -rf libavc1394.so.0 )
( cd usr/lib ; ln -sf libavc1394.so.0.3.0 libavc1394.so.0 )
( cd usr/lib ; rm -rf librom1394.so )
( cd usr/lib ; ln -sf librom1394.so.0.3.0 librom1394.so )
( cd usr/lib ; rm -rf libavc1394.so )
( cd usr/lib ; ln -sf libavc1394.so.0.3.0 libavc1394.so )
